import pymel.core as pa
import maya.cmds as cmds
import maya.OpenMaya as om
import traceback

def twoEquallySpacedButtons(label1, command1, label2, command2):
    """ Given the labels for two buttons, and corresponding commands, creates a
    form layout, and puts the buttons into it so that they stretch across it, 
    evenly distributed."""
    buttonFormLayout = pa.formLayout()
    button1 = pa.button(label=label1, command=command1)
    button2 = pa.button(label=label2, command=command2)

    buttonFormLayout.attachForm(button1, "left", 2)
    buttonFormLayout.attachForm(button1, "top", 2)
    buttonFormLayout.attachForm(button1, "bottom", 2)
    buttonFormLayout.attachPosition(button1, "right", 2, 50)
    buttonFormLayout.attachForm(button2, "right", 2)
    buttonFormLayout.attachForm(button2, "top", 2)
    buttonFormLayout.attachForm(button2, "bottom", 2)
    buttonFormLayout.attachPosition(button2, "left", 2, 50)

    pa.setParent('..')

def isTSM3Node(node):
    """ Safe function for determining if a node is a TSM3 node. Handles degenerate nodes that can
    cause PyMel to throw an exception."""
    try:
        return node.hasAttr('TSM3NodeId')
    except KeyError:
        return False

def allTSM3Nodes():
    """ Returns a list of all TSM3Nodes in the scene"""
    return [node for node in pa.ls() if isTSM3Node(node)]

def allRootJoints():
    """ Returns a list of all root joints in the scene"""
    return [node for node in pa.ls(type='joint')
            if node.hasAttr('TSM3NodeId') and node.TSM3NodeId.get() == 'TSM3WorldJoint']

def importAllTSM3References():
    """ Imports all files from which TSM3 root joints are referenced"""
    for rootJoint in allRootJoints():
        if rootJoint.isReferenced():
            rootJoint.referenceFile().importContents()

def allJointDescendentsOfJoint(joint):
    """ Gets all descendents of a joint that are joints"""
    return [node for node in joint.listRelatives(ad=True) if node.type()==u'joint']

def allNonJointDescendentsOfJoint(joint):
    """ Gets all descendents of a joint that are not joints"""
    return [node for node in joint.listRelatives(ad=True) if node.type()!=u'joint']

def allHierarchyJoints():
    """ Gets all joints in the scene that are descendents of a root joint"""
    result=[]

    for rootJoint in allRootJoints():
        result=result+allJointDescendentsOfJoint(rootJoint)

    return result

def allHierarchyNonJoints():
    """ Gets all nodes in the scene that are descendents of a root joint, but not joints"""
    result=[]

    for rootJoint in allRootJoints():
        result=result+allNonJointDescendentsOfJoint(rootJoint)

    return result

def allHierarchyNodes():
    """ Gets all nodes in the scene that are descendents of a root joint, but not joints"""
    result=[]

    for rootJoint in allRootJoints():
        result=result+rootJoint.listRelatives(ad=True)

    return result

def cleanHierarchies():
    """ Delete all nodes under a root joint that are not joints"""
    pa.delete(allHierarchyNonJoints())

def cleanScene():
    """ Delete all TSM3 nodes in scene that are not part of a TSM3 joint hierarchy"""
    hierarchyNodes=allHierarchyNodes()+allRootJoints()
    toBeDeleted=[]

    for node in allTSM3Nodes():
        if node not in hierarchyNodes:
            toBeDeleted.append(node)
            node.nodeState.set(2)

    pa.delete(toBeDeleted)

def moveRootJointsToWorld():
    for rootJoint in allRootJoints():
        pa.parent(rootJoint, w=True)

def bakeKeyframes(additionalNodes=[]):
    """ Bakes the keyframes of all of our joints, and any additional nodes passed in."""
    # This function is documented in the manual, so its signature should not change.
    startTime=pa.playbackOptions(ast=True, q=True)
    endTime=pa.playbackOptions(aet=True, q=True)

    # The format of the time parameter to bakeResults changed between
    # Maya 2011 and 2012, hence the following:
    if pa.about(api=True) >= 201200:
        timeParam = (startTime, endTime)
    else:
        timeParam = str(startTime) + ":" + str(endTime)

    pa.bakeResults(allRootJoints()+allHierarchyJoints()+additionalNodes, simulation=True, 
                   sampleBy=1, time=timeParam,
                   disableImplicitControl=True, preserveOutsideKeys=True, hierarchy=True,
                   sparseAnimCurveBake=False, removeBakedAttributeFromLayer=False, 
                   bakeOnOverrideLayer=False, controlPoints=False, shape=True)

def wiggleRootJoints():
    for rootJoint in allRootJoints():
        rootJoint.translate.set(rootJoint.translate.get())

def prepCharactersForExport(additionalNodes=[]):
    """ This function prepares the entire scene for export. AdditionalNodes is an optional list of nodes
    that are not our joints, but that should also have their keyframes baked."""
    # This function is documented in the manual,
    # so its signature should not change in any
    # way that would break it.
    try:
        importAllTSM3References()
        rootTempTransforms = bakeRootJointsToTempTransforms()
        moveRootJointsToWorld()
        bakeKeyframes(additionalNodes=additionalNodes)
        cleanHierarchies()
        cleanScene()
        # a bunch of hoodoo done to avoid evaluation errors. I think the
        # important bit is setting the current time, but I'm leaving the
        # other stuff in just in case.
        wiggleRootJoints()
        pa.currentTime(pa.currentTime())
        pa.refresh()
        pa.select(None)
        pa.delete(rootTempTransforms)
    except:
        raise
    finally:
        pa.select(None)
        
def bakeRootJointsToTempTransforms():
    """Bakes root joint movement to world space temp transform so that it is preserved when root joint is reparented,
    and returns a list of these temp transforms for later deletion"""
    
    def rootBakeTempTransform(rootJoint):
        trans = pa.createNode('transform')
        constraint = pa.parentConstraint(rootJoint, trans)
        return trans, constraint
        
    rootJoints = allRootJoints()    
    tempTransforms, tempConstraints = [list(tuple) for tuple
                                    in zip(*[rootBakeTempTransform(rootJoint)
                                    for rootJoint in rootJoints])]
    
    startTime=pa.playbackOptions(ast=True, q=True)
    endTime=pa.playbackOptions(aet=True, q=True)

    # The format of the time parameter to bakeResults changed between
    # Maya 2011 and 2012, hence the following:
    if pa.about(api=True) >= 201200:
        timeParam = (startTime, endTime)
    else:
        timeParam = str(startTime) + ":" + str(endTime)

    pa.bakeResults(tempTransforms, simulation=True, 
                   sampleBy=1, time=timeParam,
                   disableImplicitControl=True, preserveOutsideKeys=True, hierarchy=True,
                   sparseAnimCurveBake=False, removeBakedAttributeFromLayer=False, 
                   bakeOnOverrideLayer=False, controlPoints=False, shape=True)
                   
    pa.delete(tempConstraints)
    for transform, joint in zip(tempTransforms, rootJoints):
        pa.parentConstraint(transform, joint)
    
    return tempTransforms

def displayBakePrompt():
    """ Prompts user to confirm baking, returns button user pressed in dialog """
    def promptDialogUI():
        def bakeButton(ignored=True):
            pa.layoutDialog(dismiss="Bake")
        def cancelButton(ignored=True):
            pa.layoutDialog(dismiss="Cancel")
            
        formLayout = pa.setParent(q=True)
        pa.formLayout(formLayout, e=True, width=350)

        columnLayout = pa.columnLayout(adjustableColumn=True)
        pa.text(label="Are you sure you want to bake?", font="boldLabelFont", align="center")
        pa.text(label="Warning: Baking is not undoable.  We strongly recommend that you save before you " \
                      "proceed, and use 'Save As' after baking to avoid overwriting this version of your file.",
                wordWrap=True,
                align='left')
        
        pa.separator(style='none', height=8)
        twoEquallySpacedButtons("Bake", bakeButton, "Cancel", cancelButton)
        
        pa.formLayout(formLayout, e=True, attachForm=[(columnLayout, "left", 8),
                                                      (columnLayout, "right", 8),
                                                      (columnLayout, "top", 8),
                                                      (columnLayout, "bottom", 8)])
        
    dismissString = pa.layoutDialog(ui=promptDialogUI)

    return dismissString

def shouldWeBake():
    """ Displays a dialog making sure the user wants to bake. If baking should proceed, returns True """
    dismissString = displayBakePrompt()

    return (dismissString == "Bake")

def closeBakingReassuranceProgressWindow():
    """ Closes the 'Baking...' window """
    if pa.window("TSM3BakingReassuranceWindow", exists=True):
        pa.deleteUI("TSM3BakingReassuranceWindow")

def showBakingReassuranceProgressWindow():
    """ Displays a 'Baking...' window """
    closeBakingReassuranceProgressWindow()
    
    theWindow = pa.window('TSM3BakingReassuranceWindow', title='Baking...', sizeable=False)
    pa.columnLayout(adjustableColumn=True, columnOffset=["both", 8])
    pa.separator(style='none', height=8)
    pa.text(label='TSM3 is baking your scene... Please stand by...')
    pa.separator(style='none', height=8)
    pa.showWindow(theWindow)
    pa.refresh()

def bakeAction(ignored=None):
    """ Prepares the scene for export."""
    if not shouldWeBake():
        return

    showBakingReassuranceProgressWindow()
    try:
        prepCharactersForExport()
    except Exception as e:
        traceback.print_exc()
        om.MGlobal.displayError("An unexpected error occurred during scene baking. Your scene file is likely " \
                                "in a bad state now, so we recommend you restore to a previous saved version. " \
                                "Printing details to the script editor.")
        return
    finally:
        closeBakingReassuranceProgressWindow()
